{$APPTYPE CONSOLE}
{$O-}
program Project1;
uses
  Classes,
  typinfo,
  PAXCOMP_CONSTANTS,
  PAXCOMP_SYS,
  PaxCompiler, PaxProgram, PaxRegister, IMPORT_Common;

type
  TCommentRec = class
  public
    Id: Integer;
    Comment: String;
    FullName: String;
  end;

  TCommentList = class(TTypedList)
  private
    function GetRecord(I: Integer): TCommentRec;
  public
    function Add: TCommentRec;
    property Records[I: Integer]: TCommentRec read GetRecord; default;
  end;

  THandler = class
    CommentList: TCommentList;
    constructor Create;
    destructor Destroy; override;
    procedure DoComment(Sender: TPaxCompiler;
                        const AComment: String;
                        const AContext: String;
                        CommentedTokens: TStrings);
    procedure Dump;
  end;

function TCommentList.GetRecord(I: Integer): TCommentRec;
begin
  result := TCommentRec(L[I]);
end;

function TCommentList.Add: TCommentRec;
begin
  result := TCommentRec.Create;
  L.Add(result);
end;

constructor THandler.Create;
begin
  inherited;
  CommentList := TCommentList.Create;
end;

destructor THandler.Destroy;
begin
  CommentList.Free;
  inherited;
end;

function GetFullName(TokenList: TStrings; const AContext: String; var Id: Integer): String;
  var
    I: Integer;
  function NextToken: String;
  begin
    result := '';
    Inc(I);
    if I < TokenList.Count then
      result := TokenList[I];
  end;
var
  S: String;
begin
  result := '';
  I := -1;
  result := NextToken;
  Id := 0;

  if StrEql(result, 'class') then
  begin
    result := NextToken;
  end
  else if StrEql(result, 'unit') then
  begin
    result := NextToken;
    Exit;
  end;

  if StrEql(result, 'function') then
  begin
    result := NextToken;
    S := NextToken;
    if S = '.' then
      result := result + '.' + NextToken;
  end
  else if StrEql(result, 'procedure') then
  begin
    result := NextToken;
    S := NextToken;
    if S = '.' then
      result := result + '.' + NextToken;
  end
  else if StrEql(result, 'constructor') then
  begin
    result := NextToken;
    S := NextToken;
    if S = '.' then
      result := result + '.' + NextToken;
  end
  else if StrEql(result, 'destructor') then
  begin
    result := NextToken;
    S := NextToken;
    if S = '.' then
      result := result + '.' + NextToken;
  end
  else if StrEql(result, 'property') then
  begin
    result := NextToken;
  end;

  if result = '' then
    Exit;
  if AContext <> '' then
    result := AContext + '.' + result;

  Id := Integer(TokenList.Objects[I]);
end;

procedure THandler.DoComment(Sender: TPaxCompiler;
                             const AComment: String;
                             const AContext: String;
                             CommentedTokens: TStrings);
var
  AFullName: String;
  AnId: Integer;
begin
  AFullName := GetFullName(CommentedTokens, AContext, AnId);

  with CommentList.Add do
  begin
    Comment := AComment;
    FullName := AFullName;
    if AnId = 0 then
      Id := Sender.LookupId(AFullName)
    else
      Id := AnId;
  end;
end;

procedure THandler.Dump;
var
  I: Integer;
begin
  for I := 0 to CommentList.Count - 1 do
  with CommentList[I] do
  begin
    writeln(Comment);
    writeln(FullName);
    writeln(Id);
  end;
end;

var
  PaxCompiler1: TPaxCompiler;
  PaxProgram1: TPaxProgram;
  PaxPascalLanguage1: TPaxPascalLanguage;
  I: Integer;
  Handler: THandler;
begin
  Handler := THandler.Create;
  PaxCompiler1 := TPaxCompiler.Create(nil);
  PaxProgram1 := TPaxProgram.Create(nil);
  PaxPascalLanguage1 := TPaxPascalLanguage.Create(nil);

  PaxCompiler1.OnComment := Handler.DoComment;
  try
    PaxCompiler1.RegisterLanguage(PaxPascalLanguage1);
    PaxCompiler1.AddModule('1', 'Pascal');
    PaxCompiler1.AddCodeFromFile('1', 'script2.txt');
    if PaxCompiler1.Compile(PaxProgram1) then
    begin
      Handler.Dump;
    end
    else
    begin
      writeln(PaxCompiler1.ErrorMessage[0]);
      writeln(PaxCompiler1.ErrorLineNumber[0]);
      writeln(PaxCompiler1.ErrorLine[0]);
    end;

    for I:=0 to PaxCompiler1.WarningCount - 1 do
    begin
      writeln(PaxCompiler1.WarningMessage[0]);
      writeln(PaxCompiler1.WarningLineNumber[0]);
      writeln(PaxCompiler1.WarningLine[0]);
    end;

  finally
    PaxCompiler1.Free;
    PaxProgram1.Free;
    PaxPascalLanguage1.Free;
    Handler.Free;
  end;

  writeln('Press any key...');
  Readln;
end.
